unit View.OrderList;

interface

uses
  System.SysUtils, System.Classes, JS, Web, WEBLib.Graphics, WEBLib.Controls,
  WEBLib.Forms, WEBLib.Dialogs, WEBLib.StdCtrls, XData.Web.Client, WEBLib.ExtCtrls,
  Data.DB, XData.Web.Dataset, VCL.TMSFNCTypes, VCL.TMSFNCGrid, VCL.TMSFNCGridData,
  VCL.TMSFNCUtils, VCL.TMSFNCGraphics, VCL.TMSFNCGraphicsTypes,
  VCL.TMSFNCGridCell, VCL.TMSFNCGridOptions, VCL.TMSFNCCustomComponent,
  VCL.TMSFNCCustomGrid, VCL.TMSFNCGridDatabaseAdapter, VCL.TMSFNCCustomControl,
  VCL.TMSFNCCustomScrollControl, WEBLib.DB, XData.Web.JsonDataset, Vcl.StdCtrls,
  Vcl.Controls, ConnectionModule;

type
  TFOrderList = class(TWebForm)
    lblEntries: TWebLabel;
    lblEntries2: TWebLabel;
    edtSearch: TWebEdit;
    edtCompanyName: TWebEdit;
    edtID: TWebEdit;
    pnlMessage: TWebPanel;
    lblMessage: TWebLabel;
    btnCloseNotification: TWebButton;
    btnFilter: TWebButton;
    XDataWebClient1: TXDataWebClient;
    wdsCustomers: TWebDataSource;
    xdwdsCustomers: TXDataWebDataSet;
    xdwdsCustomersID: TIntegerField;
    xdwdsCustomersNAME: TStringField;
    xdwdsCustomersSHORT_NAME: TStringField;
    xdwdsCustomersADDRESS: TStringField;
    TMSFNCGrid1: TTMSFNCGrid;
    btnConfirm: TWebButton;
    cbCorrugatedPlate: TWebCheckBox;
    cbWebPlate: TWebCheckBox;
    procedure WebFormShow(Sender: TObject);
    procedure btnFilterClick(Sender: TObject);
    procedure cbCorrugatedPlateClick(Sender: TObject);
    procedure cbWebPlateClick(Sender: TObject);
    procedure btnConfirmClick(Sender: TObject);
    procedure edtSearchChange(Sender: TObject);
  private
    [async] procedure getCustomers();
    procedure PopulateGridManually;
    procedure ApplyFilter;
  public
  end;

var
  FOrderList: TFOrderList;

implementation

{$R *.dfm}

procedure TFOrderList.WebFormShow(Sender: TObject);
begin
  getCustomers(); // Fetch and populate the grid with customer data
end;

[async]
procedure TFOrderList.getCustomers();
var
  xdcResponse: TXDataClientResponse;
  customerList: TJSObject;
begin
  // Fetch data from XData service
  xdcResponse := await(XDataWebClient1.RawInvokeAsync('ILookupService.GetCustomers', []));
  customerList := TJSObject(xdcResponse.Result);

  // Load data into TXDataWebDataset
  xdwdsCustomers.Close;
  xdwdsCustomers.SetJsonData(customerList['data']);
  xdwdsCustomers.Open;

  // Manually populate the grid
  PopulateGridManually;
end;

procedure TFOrderList.PopulateGridManually;
var
  RowIndex: Integer;
begin
  TMSFNCGrid1.BeginUpdate;
  try
    TMSFNCGrid1.Clear; // Clear any existing data

    // Set up column headers
    TMSFNCGrid1.ColumnCount := 4;
    TMSFNCGrid1.RowCount := 1;
    TMSFNCGrid1.Cells[0, 0] := 'ID';
    TMSFNCGrid1.Cells[1, 0] := 'Name';
    TMSFNCGrid1.Cells[2, 0] := 'Short Name';
    TMSFNCGrid1.Cells[3, 0] := 'Address';

    // Populate the grid with data from the dataset
    xdwdsCustomers.First;
    RowIndex := 1;

    while not xdwdsCustomers.EOF do
    begin
      TMSFNCGrid1.RowCount := RowIndex + 1;

      TMSFNCGrid1.Cells[0, RowIndex] := xdwdsCustomers.FieldByName('ID').AsString;
      TMSFNCGrid1.Cells[1, RowIndex] := xdwdsCustomers.FieldByName('NAME').AsString;
      TMSFNCGrid1.Cells[2, RowIndex] := xdwdsCustomers.FieldByName('SHORT_NAME').AsString;
      TMSFNCGrid1.Cells[3, RowIndex] := xdwdsCustomers.FieldByName('ADDRESS').AsString;

      Inc(RowIndex);
      xdwdsCustomers.Next;
    end;

    // Auto-size columns
    TMSFNCGrid1.AutoSizeColumns;
  finally
    TMSFNCGrid1.EndUpdate;
  end;
end;

procedure TFOrderList.ApplyFilter;
var
  fd: TTMSFNCGridFilterData;
  i: Integer;
  SearchText: string;
begin
  SearchText := Trim(edtSearch.Text);

  // Remove any existing filters
  TMSFNCGrid1.RemoveFilter;


  TMSFNCGrid1.Filter.Clear;

  // Add a filter for each column
  for i := 0 to TMSFNCGrid1.ColumnCount - 1 do
  begin
    fd := TMSFNCGrid1.Filter.Add;
    fd.Column := i;
    fd.Condition := '*' + SearchText + '*'; // Match text anywhere in the cell
    fd.CaseSensitive := False; // Make the filter case-insensitive

    // Use foOR for "match any column" logic
    if i > 0 then
      fd.Operation := foOR
    else
      fd.Operation := foNONE; // First filter has no logical operation
  end;

  // Apply the filters to the grid
  TMSFNCGrid1.ApplyFilter;
end;


procedure TFOrderList.btnFilterClick(Sender: TObject);
begin
  ApplyFilter;
end;

procedure TFOrderList.edtIDChange(Sender: TObject);
begin
  cbWebPlate.Checked := False;
end;

procedure TFOrderList.cbWebPlateClick(Sender: TObject);
begin
  cbCorrugatedPlate.Checked := False;
end;

procedure TFOrderList.btnConfirmClick(Sender: TObject);
begin
  Close;
end;

procedure TFOrderList.edtSearchChange(Sender: TObject);
begin
  ApplyFilter;
end;

end.

