// Displays orders in the grid. Allows the user
// to sort the entries, filter their search, and search for a specific person.
// Authors:
// Cameron Hayes
// Mac ...

unit View.Orders;

interface

uses
  System.SysUtils, System.Generics.Collections, System.Classes, JS, Web, WEBLib.Graphics, WEBLib.Controls,
  WEBLib.Forms, WEBLib.Dialogs, WEBLib.Menus, WEBLib.ExtCtrls, WEBLib.StdCtrls,
  WEBLib.JSON, Auth.Service, XData.Web.Client, WebLib.Storage,
  ConnectionModule, App.Types, Vcl.StdCtrls, Vcl.Controls, WEBLib.DBCtrls,
  Data.DB, XData.Web.JsonDataset, XData.Web.Dataset, WEBLib.DB;

type
  TFViewOrders = class(TWebForm)
    XDataWebClient1: TXDataWebClient;
    XDataWebDataSet1: TXDataWebDataSet;
    lblEntries: TWebLabel;
    wcbPageSize: TWebComboBox;
    wcbSortBy: TWebComboBox;
    btnApply: TWebButton;
    edtSearch: TWebEdit;
    WebDataSource1: TWebDataSource;
    XDataWebDataSet1companyName: TStringField;
    XDataWebDataSet1jobName: TStringField;
    XDataWebDataSet1orderDate: TStringField;
    XDataWebDataSet1proofDue: TStringField;
    XDataWebDataSet1proofDone: TStringField;
    XDataWebDataSet1artDue: TStringField;
    XDataWebDataSet1artDone: TStringField;
    XDataWebDataSet1mountDue: TStringField;
    XDataWebDataSet1mountDone: TStringField;
    XDataWebDataSet1shipDue: TStringField;
    XDataWebDataSet1shipDone: TStringField;
    XDataWebDataSet1price: TStringField;
    XDataWebDataSet1ID: TStringField;
    XDataWebDataSet1qbRefNum: TStringField;
    XDataWebDataSet1colors: TStringField;
    btnaddOrder: TWebButton;
    btnFilters: TWebButton;
    dtpStartDate: TWebDateTimePicker;
    dtpEndDate: TWebDateTimePicker;
    lblEntries2: TWebLabel;
    XDataWebDataSet1plateDue: TStringField;
    XDataWebDataSet1plateDone: TStringField;
    XDataWebDataSet1orderType: TStringField;
    pnlMessage: TWebPanel;
    lblMessage: TWebLabel;
    btnCloseNotification: TWebButton;
    WebButton1: TWebButton;
    WebButton2: TWebButton;
    btnConfirm: TWebButton;
    wdsCustomers: TWebDataSource;
    wcbCustomers: TWebDBLookupComboBox;
    xdwdsCustomers: TXDataWebDataSet;
    xdwdsCustomersNAME: TStringField;
    xdwdsCustomersID: TIntegerField;
    xdwdsCustomersCURR_ID: TIntegerField;
    xdwdsSave: TXDataWebDataSet;
    wdsSave: TWebDataSource;
    xdwdsSaveCURR_ID: TIntegerField;
    pnlOrders: TWebPanel;
    procedure WebFormCreate(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnSearchClick(Sender: TObject);
    procedure btnaddOrderClick(Sender: TObject);
    procedure btnFiltersClick(Sender: TObject);
    procedure btnCloseNotificationClick(Sender: TObject);
    procedure WebFormShow(Sender: TObject);
    procedure btnConfirmClick(Sender: TObject);
    procedure wcbCustomersChange(Sender: TObject);
  private
    FChildForm: TWebForm;
    procedure AddRowToTable(temp: string);
    procedure ClearTable();
    procedure GeneratePagination(TotalPages: Integer);
    function GenerateSearchOptions(): string;
    procedure orderEntry(orderInfo, customerInfo, mode: string);
    procedure HideNotification();
    procedure ShowNotification(Notification: string);
    procedure ShowOrderListForm();
    [async] procedure Search(searchOptions: string);
    [async] procedure GetOrders(searchOptions: string);
    [async] procedure getUser();
    var
      PageNumber: integer;
      PageSize: integer;
      TotalPages: integer;
      StartDate: string;
      EndDate: string;
      OrderBy: string;
      Caller: string;
      filters: boolean;
      info: string;
  public
    class function CreateForm(AElementID, Info: string): TWebForm;

  end;

var
  FViewOrders: TFViewOrders;

implementation

uses
  XData.Model.Classes, View.Main, View.OrderList;

{$R *.dfm}

procedure TFViewOrders.WebFormCreate(Sender: TObject);
// Initializes important values:
// PageNumber: What page number the user is on IE 1: 1-10, 2: 11-20 etc
// TotalPages: Total number of pages returned from the search.
// PageSize: Number of entries per page.
var
  today: TDateTime;
begin
  DMConnection.ApiConnection.Connected := True;
  PageNumber := 1;
  TotalPages := 1; // Initial total pages
  wcbPageSize.Text := '10';
  wcbSortBy.Text := 'PROOF';
  //today := TDateTime.Today;
  dtpStartDate.Date := 0;
  dtpEndDate.Date := 0;
  getOrders(GenerateSearchOptions());
end;

procedure TFViewOrders.WebFormShow(Sender: TObject);
begin
  console.log(info);
  if info <> '' then
    ShowNotification(info)
  else
    HideNotification();
end;

procedure TFViewOrders.getUser();
var
  xdcResponse: TXDataClientResponse;
  userList, user: TJSObject;
  data: TJSArray;
begin
  xdcResponse := await(XDataWebClient1.RawInvokeAsync('ILookupService.GetUsers',
        [JS.toString(AuthService.TokenPayload.Properties['user_name'])]));
  userList :=  TJSObject(xdcResponse.Result);
  data := TJSArray(userList['data']);
  user := TJSObject(data[0]);
end;

class function TFViewOrders.CreateForm(AElementID, Info: string): TWebForm;
var
  localInfo: string;
begin
  localInfo := info;
  Application.CreateForm(TFViewOrders, AElementID, Result,
    procedure(AForm: TObject)
    begin
      with TFViewOrders(AForm) do
      begin
        TFViewOrders(AForm).info := LocalInfo;
      end;
    end
  );
end;


procedure TFViewOrders.ShowOrderListForm();
var
  OrdersPanel: TJSHTMLElement;
begin
  // Get the panel element for orders
  OrdersPanel := TJSHTMLElement(document.getElementById('pnl_orders'));

  // Hide the panel before loading the form
  if Assigned(OrdersPanel) then
    OrdersPanel.style.setProperty('display', 'none');

  // Create the order list form, passing the ElementID
  TFOrderList.CreateForm(pnlOrders.ElementID);

  // used to manage Back button handling to close subform
  window.location.hash := 'subform';

  newform.ShowModal(
    procedure(AValue: TModalResult)
    begin
      if newForm.edtID.Text <> '' then
        orderEntry('', newForm.edtID.Text, 'ADD');
    end
  );
end;





procedure TFViewOrders.AddRowToTable(temp: string);
// Adds rows to the table
// PhoneNumber: phone number of the location
// Caller: phone number of the caller
// Duration: duration of the call
// Transcript: transcription of the recording
// MediaUrl: Link to the recording
var
  NewRow, Cell, P, Button, Audio: TJSHTMLElement;
  colorObject: TJSObject;
  colorList: TJSArray;
  colorListJSON: TJSONArray;
  colors: integer;
  ClickHandler: TJSFunction;
  id, strColorList: string;
begin
  NewRow := TJSHTMLElement(document.createElement('tr'));
  NewRow.Attrs['orderType'] := XDataWebDataSet1orderType.Value;
  NewRow.addEventListener('click', procedure
  begin
    orderEntry(NewRow.Attrs['id'], '', 'EDIT');
  end);

  // Order ID Cell
  NewRow.Attrs['id'] := XDataWebDataSet1ID.Value;
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Order ID');
  if XDataWebDataSet1ID.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1ID.Value;
  NewRow.appendChild(Cell);

  // Company Name Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Company Name');
  if XDataWebDataSet1companyName.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1companyName.Value;
  NewRow.appendChild(Cell);

  // Job Name Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Job Name');
  if XDataWebDataSet1jobName.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1jobName.Value;
  NewRow.appendChild(Cell);

  // Order Date Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Order Date');
  if XDataWebDataSet1orderDate.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1orderDate.Value;
  NewRow.appendChild(Cell);

  // Proof Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Proof Due');
  if XDataWebDataSet1proofDue.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1proofDue.Value;
  NewRow.appendChild(Cell);

  // Proof Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Proof Done');
  if XDataWebDataSet1proofDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1proofDone.Value;
  NewRow.appendChild(Cell);

  // Art Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Art Due');
  if XDataWebDataSet1artDue.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1artDue.Value;
  NewRow.appendChild(Cell);

  // Art Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Art Done');
  if XDataWebDataSet1artDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1artDone.Value;
  NewRow.appendChild(Cell);

  // Plate Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Plate Due');
  if XDataWebDataSet1plateDue.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1plateDue.Value;
  NewRow.appendChild(Cell);

  // Plate Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Plate Done');
  if XDataWebDataSet1plateDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1plateDone.Value;
  NewRow.appendChild(Cell);

  // Mount Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Mount Due');
  if XDataWebDataSet1mountDue.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1mountDue.Value;
  NewRow.appendChild(Cell);

  // Mount Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Mount Done');
  if XDataWebDataSet1mountDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1mountDone.Value;
  NewRow.appendChild(Cell);

  // Ship Due Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Ship Due');
  if XDataWebDataSet1shipDue.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1shipDue.Value;
  NewRow.appendChild(Cell);

  // Ship Done Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Ship Done');
  if XDataWebDataSet1shipDone.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1shipDone.Value;
  NewRow.appendChild(Cell);

  // Price Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Price');
  if XDataWebDataSet1price.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1price.Value;
  NewRow.appendChild(Cell);

  // QuickBooks Reference Number Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Quickbooks Reference Number');
  if XDataWebDataSet1qbRefNum.Value = '' then
    Cell.innerText := 'None'
  else
    Cell.innerText := XDataWebDataSet1qbRefNum.Value;
  NewRow.appendChild(Cell);

  // Colors Cell
  Cell := TJSHTMLElement(document.createElement('td'));
  Cell.setAttribute('data-label', 'Colors');
  if XDataWebDataSet1colors.Value = '' then
    Cell.innerText := '0'
  else
  begin
    colorObject := TJSObject(TJSJSON.parse(XDataWebDataSet1colors.Value));
    colorList := TJSArray(colorObject['items']);
    colors := colorList.flength;
    Cell.innerText := IntToStr(colors);
  end;
  NewRow.appendChild(Cell);

  // Appends new rows to the table body
  TJSHTMLElement(document.getElementById('tblPhoneGrid').getElementsByTagName('tbody')[0]).appendChild(NewRow);
end;

procedure TFViewOrders.GeneratePagination(TotalPages: Integer);
// Generates pagination for the table.
// TotalPages: Total amount of pages generated by the search
var
  PaginationElement, PageItem, PageLink: TJSHTMLElement;
  I, Start, Finish: Integer;
begin
  PaginationElement := TJSHTMLElement(document.getElementById('pagination'));
  PaginationElement.innerHTML := ''; // Clear existing pagination

  // Previous Button
  PageItem := TJSHTMLElement(document.createElement('li'));
  PageItem.className := 'page-item';
  if PageNumber = 1 then
    PageItem.classList.add('disabled');
  PageLink := TJSHTMLElement(document.createElement('a'));
  PageLink.className := 'page-link';
  PageLink.innerText := 'Previous';
  PageLink.setAttribute('href', 'javascript:void(0)');
  PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
  begin
    if PageNumber > 1 then
    begin
      Dec(PageNumber);
      GetOrders(GenerateSearchOptions());
    end;
  end);
  PageItem.appendChild(PageLink);
  PaginationElement.appendChild(PageItem);

  // Page Numbers

  if TotalPages <= 7 then
  begin
    for I := 1  to 7 do
    begin
      if I <= TotalPages then
      begin
        PageItem := TJSHTMLElement(document.createElement('li'));
        PageItem.className := 'page-item';
        if I = PageNumber then
          PageItem.classList.add('selected-number'); // Add the selected-number class
        PageLink := TJSHTMLElement(document.createElement('a'));
        PageLink.className := 'page-link';
        PageLink.innerText := IntToStr(I);
        PageLink.setAttribute('href', 'javascript:void(0)');
        PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
        var
          PageNum: Integer;
        begin
          PageNum := StrToInt((Event.currentTarget as TJSHTMLElement).innerText);
          PageNumber := PageNum;
          GetOrders(GenerateSearchOptions());
        end);
        PageItem.appendChild(PageLink);
        PaginationElement.appendChild(PageItem);
      end;
    end;
  end
  else
  begin
    if PageNumber <= 4 then
    // If page number is low enough no early elipsis needed
    Begin
      Start := 2;
      Finish := 5;
    End
    else if (PageNumber >= (TotalPages - 3)) then
    // If page number is high enough no late elipsis needed
    begin
      Start := TotalPages - 3;
      Finish := TotalPages - 1;
    end
    else
    begin
      Start := PageNumber - 1;
      Finish := PageNumber + 1;
    end;

    PageItem := TJSHTMLElement(document.createElement('li'));
    PageItem.className := 'page-item';
    if 1 = PageNumber then
      PageItem.classList.add('selected-number'); // Add the selected-number class
    PageLink := TJSHTMLElement(document.createElement('a'));
    PageLink.className := 'page-link';
    PageLink.innerText := '1';
    PageLink.setAttribute('href', 'javascript:void(0)');
    PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
    var
      PageNum: Integer;
    begin
      PageNum := StrToInt((Event.currentTarget as TJSHTMLElement).innerText);
      PageNumber := PageNum;
      GetOrders(GenerateSearchOptions());
    end);
    PageItem.appendChild(PageLink);
    PaginationElement.appendChild(PageItem);

    // Adds Elipse to pagination if page number is too big
    if PageNumber > 4 then
    begin
      PageItem := TJSHTMLElement(document.createElement('li'));
      PageItem.className := 'page-item';
      PageItem.classList.add('disabled');
      PageLink := TJSHTMLElement(document.createElement('a'));
      PageLink.className := 'page-link';
      PageLink.innerText := '...';
      PageLink.setAttribute('href', 'javascript:void(0)');
      PageItem.appendChild(PageLink);
      PaginationElement.appendChild(PageItem);
    end;


    // Adds Page, page - 1, and page + 1 to pagination
    for I := Start  to Finish do
    begin
      if ( I > 1) and (I < TotalPages) then
      begin
        PageItem := TJSHTMLElement(document.createElement('li'));
        PageItem.className := 'page-item';
        if I = PageNumber then
          PageItem.classList.add('selected-number'); // Add the selected-number class
        PageLink := TJSHTMLElement(document.createElement('a'));
        PageLink.className := 'page-link';
        PageLink.innerText := IntToStr(I);
        PageLink.setAttribute('href', 'javascript:void(0)');
        PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
        var
          PageNum: Integer;
        begin
          PageNum := StrToInt((Event.currentTarget as TJSHTMLElement).innerText);
          PageNumber := PageNum;
          GetOrders(GenerateSearchOptions());
        end);
        PageItem.appendChild(PageLink);
        PaginationElement.appendChild(PageItem);
      end;
    end;

    // adds ellipse if number is too small
    if PageNumber < TotalPages - 4 then
    begin
      PageItem := TJSHTMLElement(document.createElement('li'));
      PageItem.className := 'page-item';
      PageItem.classList.add('disabled');
      PageLink := TJSHTMLElement(document.createElement('a'));
      PageLink.className := 'page-link';
      PageLink.innerText := '...';
      PageLink.setAttribute('href', 'javascript:void(0)');
      PageItem.appendChild(PageLink);
      PaginationElement.appendChild(PageItem);
    end;

    if TotalPages <> 1 then
    begin
      PageItem := TJSHTMLElement(document.createElement('li'));
      PageItem.className := 'page-item';
      if TotalPages = PageNumber then
            PageItem.classList.add('selected-number');
      PageLink := TJSHTMLElement(document.createElement('a'));
      PageLink.className := 'page-link';
      PageLink.innerText := IntToStr(TotalPages);
      PageLink.setAttribute('href', 'javascript:void(0)');
      PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
      var
        PageNum: Integer;
      begin
        PageNum := StrToInt((Event.currentTarget as TJSHTMLElement).innerText);
        PageNumber := PageNum;
        GetOrders(generateSearchOptions());
      end);
    end;
    PageItem.appendChild(PageLink);
    PaginationElement.appendChild(PageItem);
  end;

  // Next Button
  PageItem := TJSHTMLElement(document.createElement('li'));
  PageItem.className := 'page-item';
  if PageNumber = TotalPages then
    PageItem.classList.add('disabled');
  PageLink := TJSHTMLElement(document.createElement('a'));
  PageLink.className := 'page-link';
  PageLink.innerText := 'Next';
  PageLink.setAttribute('href', 'javascript:void(0)');
  PageLink.addEventListener('click', procedure(Event: TJSMouseEvent)
  begin
    if PageNumber < TotalPages then
    begin
      Inc(PageNumber);
      GetOrders(GenerateSearchOptions());
    end;
  end);
  PageItem.appendChild(PageLink);
  PaginationElement.appendChild(PageItem);

end;


procedure TFViewOrders.GetOrders(searchOptions: string);
// Retrieves list of orders from the server that meet the criteria searchOptions
// searchOptions: information about how to generate the SQL statement based on
// user input.
var
  xdcResponse: TXDataClientResponse;
  orderList : TJSObject;
  i: integer;
  data: TJSArray;
  order: TJSObject;
  callListLength: integer;
  tempString: string;

begin
  if PageNumber > 0 then
  begin
    asm
      startSpinner();
    end;
    xdcResponse := await(XDataWebClient1.RawInvokeAsync('ILookupService.GetOrders',
        [searchOptions]));
    orderList :=  TJSObject(xdcResponse.Result);
    data := TJSArray(orderList['data']);
    callListLength := integer(orderList['count']);
    ClearTable();
    XDataWebDataSet1.Close;
    XDataWebDataSet1.SetJsonData(orderList['data']);
    XDataWebDataSet1.Open;
    asm
      setTimeout(endSpinner, 2000);
    end;
    for i := 0 to data.Length - 1 do
    begin
      order := TJSObject(data[i]);
      AddRowToTable(tempString);
      XDataWebDataSet1.Next;
    end;
    TotalPages := (callListLength + PageSize - 1) div PageSize;
    if callListLength = 0 then
    begin
      lblEntries.Caption := 'No entries found';
      lblEntries2.Caption := 'No entries found';
    end
    else if (PageNumber * PageSize) < callListLength then
    begin
      lblEntries.Caption := 'Showing entries ' + IntToStr((PageNumber - 1) * PageSize + 1) +
                            ' - ' + IntToStr(PageNumber * PageSize) +
                             ' of ' + IntToStr(callListLength);
      lblEntries2.Caption := 'Showing entries ' + IntToStr((PageNumber - 1) * PageSize + 1) +
                            ' - ' + IntToStr(PageNumber * PageSize) +
                             ' of ' + IntToStr(callListLength);
    end
    else if (PageNumber * PageSize) >= callListLength then
    begin
       lblEntries.Caption := 'Showing entries ' + IntToStr((PageNumber - 1) * PageSize + 1) +
                            ' - ' + IntToStr(callListLength) +
                             ' of ' + IntToStr(callListLength);
      lblEntries2.Caption := 'Showing entries ' + IntToStr((PageNumber - 1) * PageSize + 1) +
                            ' - ' + IntToStr(callListLength) +
                             ' of ' + IntToStr(callListLength);
    end;
    GeneratePagination(TotalPages);
  end;
end;

procedure TFViewOrders.btnaddOrderClick(Sender: TObject);
begin
  //getCustomers();
  ShowOrderListForm();
end;

procedure TFViewOrders.orderEntry(orderInfo, customerInfo, mode: string);
begin
  FViewMain.ViewOrderEntry(orderInfo, customerInfo, mode);
end;

procedure TFViewOrders.btnApplyClick(Sender: TObject);
// Button that effectively functions as a GetOrders() button
var
searchOptions: string;
begin
  PageNumber := 1;
  PageSize := StrToInt(wcbPageSize.Text);
  OrderBy := wcbSortBy.Text;
  searchOptions :=  '&pagenumber=' + IntToStr(PageNumber) +
                    '&pagesize=' + IntToStr(PageSize) +
                    '&orderby=' + OrderBy +
                    '&startDate=' + FormatDateTime('yyyy/mm/dd', dtpStartDate.Date) +
                    '&endDate=' + FormatDateTime('yyyy/mm/dd', dtpEndDate.Date) +
                    '&filterType=' + wcbSortBy.Text;
  GetOrders(searchOptions);
end;

procedure TFViewOrders.btnCloseNotificationClick(Sender: TObject);
begin
  HideNotification();
end;

procedure TFViewOrders.btnConfirmClick(Sender: TObject);
begin
  //orderEntry('', 'ADD');
end;

procedure TFViewOrders.btnFiltersClick(Sender: TObject);
var
filterSection: TJSHTMLElement;
begin
  if btnFilters.Caption = 'Show Filters' then
  begin
    btnFilters.Caption := 'Hide Filters';
    filterSection := TJSHTMLElement(document.getElementById('filter_info'));
    if Assigned(filterSection) then
      filterSection.style.setProperty('display', 'block');
  end
  else
  begin
    btnFilters.Caption := 'Show Filters';
    filterSection := TJSHTMLElement(document.getElementById('filter_info'));
    if Assigned(filterSection) then
      filterSection.style.setProperty('display', 'none');
  end;
end;

procedure TFViewOrders.Search(searchOptions: string);
// Search method that searches the database for a specific phone number
var
  xdcResponse: TXDataClientResponse;
  callList : TJSObject;
  i: integer;
  data: TJSArray;
  call: TJSObject;
  callListLength: integer;

begin
  if PageNumber > 0 then
  begin
    xdcResponse := await(XDataWebClient1.RawInvokeAsync('ILookupService.Search',
        [searchOptions]));
    callList :=  TJSObject(xdcResponse.Result);
    data := TJSArray(callList['data']);
    callListLength := integer(callList['count']);
    ClearTable();
    for i := 0 to data.Length - 1 do
    begin
      call := TJSObject(data[i]);
      {AddRowToTable(string(call['toNumber']), string(call['fromNumber']), string(call['dateCreated']),
      string(call['duration']), string(call['transcription']), string(call['mediaUrl']));}
    end;
    TotalPages := (callListLength + PageSize - 1) div PageSize;
    lblEntries.Caption := 'Showing entries for phone number: ' + searchOptions;
  end;
end;

procedure TFViewOrders.btnSearchClick(Sender: TObject);
// orders Search method
begin
  Search(edtSearch.Text);
end;

procedure TFViewOrders.ClearTable();
// clears the table
var
  tbody: TJSHTMLElement;
begin
  tbody := TJSHTMLElement(document.getElementById('tblPhoneGrid').getElementsByTagName('tbody')[0]);
  tbody.innerHTML := '';
end;

function TFViewOrders.GenerateSearchOptions(): string;
// Generates searchOptions for GetOrders.
var
searchOptions: string;
begin
  //PageNumber := 1;
  PageSize := StrToInt(wcbPageSize.Text);
  OrderBy := wcbSortBy.Text;
  searchOptions :=  '&pagenumber=' + IntToStr(PageNumber) +
                    '&pagesize=' + IntToStr(PageSize) +
                    '&orderby=' + OrderBy;
  if btnFilters.Caption = 'Hide Filters' then
    begin
      searchOptions := searchOptions +
                    '&startDate=' + FormatDateTime('yyyy/mm/dd', dtpStartDate.Date) +
                    '&endDate=' + FormatDateTime('yyyy/mm/dd', dtpEndDate.Date) +
                    '&filterType=' + wcbSortBy.Text;
    end;
  Result := searchOptions;
end;

procedure TFViewOrders.HideNotification;
begin
  pnlMessage.ElementHandle.hidden := True;
  info := '';
end;

procedure TFViewOrders.ShowNotification(Notification: string);
begin
  if Notification <> '' then
  begin
    console.log('hit');
    lblMessage.Caption := Notification;
    pnlMessage.ElementHandle.hidden := False;
  end;
end;

procedure TFViewOrders.wcbCustomersChange(Sender: TObject);
begin
  console.log(wcbCustomers.Value);
end;

end.

